import workerUrl from "./worker.js";
class ReferencedObject {
  constructor(instance) {
    this.instance = instance;
  }

  static set referencedObject(obj) {
    this.instance = obj;
  }

  static get referencedObject() {
    return this.instance;
  }
}


function ThriftConfigure() {
  this.str_engine_ini_filename = "./ini/ise.ini";
  this.str_ca_crt_filename = "./keys/CA.pem";
  this.str_client_crt_filename = "./keys/client.crt";
  this.str_client_key_filename = "./keys/client.key";
  this.str_server_crt_filename = "./keys/client.crt";
  this.str_server_key_filename = "./keys/client.key";
  this.str_key_ciphers = "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH";

  this.str_socket_tcp_host_cc = "localhost";
  this.i_socket_tcp_port_cc = 9191;
  this.str_socket_u_domain_cc = "";
  this.b_abstract_namespace_cc = false;

  this.b_use_ssl = false;
  this.b_use_zlib = false;

  this.str_type_transport = "websocket";
  this.str_type_protocol = "json";
  this.str_type_server = "threaded";
  this.str_path_transport = "/service";

  this.i_workers = 1;
  this.i_limit_string_size = 0;
  this.i_limit_container_size = 0;
}

ThriftConfigure.prototype = {};

class InputServicePanelClient {
  constructor(filename, uid) {
    this.eventHandler = null;
    this.eventHandlerAttachment = null;
    this.filename = filename;
    this.uid = uid;
    this.instanceMaps = new Map();
  }

  InstallEventHandler(handler, attachment) {
    this.eventHandler = handler;
    this.eventHandlerAttachment = attachment;
  }

  async AcquirePanel(type, filename, uid) {
    if (type === "ini") {
      let result = await GetAllThriftSettings("runtime", filename);
      type = result["type"];
    }

    switch (type) {
      case "thrift":
        return this.AcquireThriftInstance(type, filename, uid);
      default:
        throw new Error(
            "The type: [" + type + "] is NOT SUPPORTED");
    }
  }

  DestroyPanel(type, client) {
    switch (type) {
      case "thrift":
        this.DestroyThriftInstance(type, client);
        break;
      default:
        throw new Error(
            "The type: [" + type + "] is NOT SUPPORTED");
    }
  }

  AcquireThriftInstance(type, filename, uid) {
    let instance = this.instanceMaps.get(new Map(filename, uid));
    if (!instance) {
      instance = new InputServicePanelThriftClient(filename, uid);
      this.instanceMaps.set(new Map(filename, uid), instance);
    }

    return instance;
  }

  DestroyThriftInstance(type, client) {
    let keys = this.instanceMaps.keys();
    for (let key of keys) {
      if (this.instanceMaps.get(key) === client) {
        this.instanceMaps.delete(key);
        break;
      }
    }
  }


  GetAllThriftSettings(key, file) {
    return new Promise((resolve, reject) => {
      let text;
      let settings = [];
      let xhr = new XMLHttpRequest();
      xhr.open('GET', file);
      xhr.send();
      xhr.onload = () => {
        try {
          text = JSON.parse(xhr.response);
          if (text[key]) {
            for (let member in text[key]) {
              settings[member] = text[key][member];
            }
          }

          resolve(settings);
        } catch (e) {
          reject();
        }
      };
      xhr.onerror = () => {
        resolve("");
      }
    })
  }

  GetThriftSettings(prefix, result, s_key_name) {
    let value = result[s_key_name];
    if (value != null && value.length() > 0 && value.charAt(0) != '/' && value.charAt(0) != '\\') {
      return prefix + value;
    } else if (value != null) {
      return value;
    }
    return "";
  }

}


class InputServicePanelThriftClient extends InputServicePanelClient {
  TIMEOUT_TRANSPORT_OPEN = 1000 * 1000 * 60;

  transport = null;
  client = null;

  transport0 = null;
  client0 = null;

  eventHandlerThreadFlag = false;
  eventhandlerThread = null;
  constructor(filename, uid) {
    super(filename, uid);

    this.t_configure = new ThriftConfigure();
    this.GetAllThriftSettings("thrift", filename).then(async result => {

      let prefix = result["prefix"];
      this.t_configure.str_ca_crt_filename = this.GetThriftSettings(prefix, result, "ca_crt_filename");
      this.t_configure.str_client_crt_filename = this.GetThriftSettings(prefix, result, "client_crt_filename");
      this.t_configure.str_client_key_filename = this.GetThriftSettings(prefix, result, "client_key_filename");

      this.t_configure.str_key_ciphers = result["key_ciphers"];
      this.t_configure.str_socket_tcp_host_cc = result["socket_tcp_host_cc"];
      this.t_configure.str_socket_u_domain_cc = result["KEY_NAME_SOCKET_U_DOMAIN_CC"];
      this.t_configure.str_path_transport = result["path_transport"];
      this.t_configure.str_type_transport = result["type_transport"];
      this.t_configure.str_type_protocol = result["type_protocol"];
      this.t_configure.i_socket_tcp_port_cc = result["socket_tcp_port_cc"];
      this.t_configure.b_abstract_namespace_cc = result["abstract_namespace"];
      this.t_configure.b_use_ssl = Boolean(result["use_ssl"]);
      this.t_configure.b_use_zlib = Boolean(result["use_zlib"]);

      let protocol = null;
      let r_transport = new ReferencedObject(this.transport);
      let r_protocol = new ReferencedObject(protocol);

      if (await this.InitializationClient(this.t_configure, r_transport, r_protocol) != 0) {
        return;
      }
      this.client = new InputServicePanel.Client(protocol);

      let protocol0 = null;

      let r_transport0 = new ReferencedObject(this.transport0);
      let r_protocol0 = new ReferencedObject(protocol0);
      if (await this.InitializationClient(this.t_configure, r_transport0, r_protocol0) != 0) {
        return;
      }
      this.client0 = new InputServicePanel.Client(protocol0);

      this.eventHandlerThreadFlag = true;
      let worker = new Worker(workerUrl);
      worker.addEventListener("message", (evt) => {
        // 3.主线程收到消息后进行处理
        if (evt.data.data === "handleEnd") {
          this.transport.close();
          this.transport0.close();
        }
      });

      // 1. 发消息给子线程
      worker.postMessage({
        eventHandlerThreadFlag: this.eventHandlerThreadFlag,
        client0: this.client0,
        eventHandler: this.eventHandler,
        eventHandlerAttachment: this.eventHandlerAttachment
      });

    });
  }

  async InitializationClient(t_configure, r_transport, r_protocol) {
    if (t_configure.str_type_protocol === "json") {
    } else if (t_configure.str_type_protocol === "multiplex") {
    } else {
      throw new Error("Unknown protocol type! " + t_configure.str_type_protocol);
    }

    if (t_configure.str_type_transport === "http") {
 
    } else {
      throw new Error("Unknown transport type! " + t_configure.str_type_transport);
    }

    if (t_configure.str_type_transport === "http" && t_configure.b_use_ssl) {
      throw new Error("SSL is not supported over http.");
    }

    let transport = null;

    //Http[s]/XHR protocol
    if (t_configure.str_type_transport === "http") {
      // Thrift.TXHRTransport
      transport = new Thrift.Transport("http://" + t_configure.str_socket_tcp_host_cc + ":"
      + t_configure.i_socket_tcp_port_cc + t_configure.str_path_transport);

    } else {//WebSocket protocol
      transport = new Thrift.TWebSocketTransport("http://" + t_configure.str_socket_tcp_host_cc + ":"
      + t_configure.i_socket_tcp_port_cc);
    }

    let tProtocol = null;
    if (t_configure.str_type_protocol === "json") {
      tProtocol = new Thrift.Protocol(transport);
    } else if (t_configure.str_type_protocol === "multiplex") {

    }

    await this.OpenTransport(transport);

    r_transport.referencedObject = transport;
    r_protocol.referencedObject = tProtocol;
    return 0;
}

  async OpenTransport(transport) {
    let i_time_sleep = 128;
    while (true) {
      try {
        if (!transport.isOpen()) {
          transport.open();
        }
        return;
      } catch (error) {
        console.error(error);
        if (i_time_sleep > this.TIMEOUT_TRANSPORT_OPEN) {
          return;
        }
        try {
          Promise.sleep = function(timeout) {
            return new Promise(resolve => {
              setTimeout(() => {
                resolve();
              }, timeout);
            });
          };

          await Promise.sleep(i_time_sleep);
        } catch (err) {
          return;
        }

        transport.close();
        i_time_sleep *= 2;
        continue;
      }
    }
  }

  finalize() {
    try {
      this.eventHandlerThreadFlag = false;
    } catch (e) {
    }
  }

  Show(name) {
    try {
      return this.client.Show(this.uid, name);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Hide(name) {
    try {
      return this.client.Hide(this.uid, name);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Page(name, page) {
    try {
      return this.client.Page(this.uid, name, page);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Move(name, x, y) {
    try {
      return this.client.Move(this.uid, name, x, y);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Skin(skin) {
    try {
      return this.client.Skin(this.uid, skin);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Mode(mode, language) {
    try {
      return this.client.Mode(this.uid, mode, language);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  Resize(name, w, h) {
    try {
      return this.client.Resize(this.uid, name, w, h);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  CheckWindowVirtual() {
    try {
      return this.client.CheckWindowVirtual(this.uid);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  KeyDown(key) {
    try {
      return this.client.KeyDown(this.uid, key);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  KeyUp(key) {
    try {
      return this.client.KeyUp(this.uid, key);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  TouchDown(name, x, y) {
    try {
      return this.client.TouchDown(this.uid, name, x, y);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  TouchUp(name, x, y) {
    try {
      return this.client.TouchUp(this.uid, name, x, y);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  TouchMove(name, x, y) {
    try {
      return this.client.TouchMove(this.uid, name, x, y);
    } catch (error) {
      console.error(error);
    }

    return -1;
  }

  AcquireEngineStat(key) {
    try {
      return this.client.AcquireEngineStat(this.uid, key);
    } catch (error) {
      console.error(error);
    }

    return null;
  }

  AcquireWindowRect(name) {
    try {
      return this.client.AcquireWindowRect(this.uid, name);
    } catch (error) {
      console.error(error);
    }

    return null;
  }

  AcquireRenderData(name) {
    try {
      return this.client.AcquireRenderData(this.uid, name);
    } catch (error) {
      console.error(error);
    }

    return null;
  }
}